from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_q", "0005_auto_20150718_1506"),
    ]

    operations = [
        migrations.AddField(
            model_name="schedule",
            name="minutes",
            field=models.PositiveSmallIntegerField(
                help_text="Number of minutes for the Minutes type",
                blank=True,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="schedule_type",
            field=models.CharField(
                max_length=1,
                choices=[
                    ("O", "Once"),
                    ("I", "Minutes"),
                    ("H", "Hourly"),
                    ("D", "Daily"),
                    ("W", "Weekly"),
                    ("M", "Monthly"),
                    ("Q", "Quarterly"),
                    ("Y", "Yearly"),
                ],
                default="O",
                verbose_name="Schedule Type",
            ),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_q", "0006_auto_20150805_1817"),
    ]

    operations = [
        migrations.CreateModel(
            name="OrmQ",
            fields=[
                (
                    "id",
                    models.AutoField(
                        primary_key=True,
                        auto_created=True,
                        verbose_name="ID",
                        serialize=False,
                    ),
                ),
                ("key", models.CharField(max_length=100)),
                ("payload", models.TextField()),
                ("lock", models.DateTimeField(null=True)),
            ],
            options={
                "verbose_name_plural": "Queued tasks",
                "verbose_name": "Queued task",
            },
        ),
    ]



# Generated by Django 6.0.4 on 2026-04-17 20:55

import django.utils.timezone
import picklefield.fields
from django.db import migrations, models

import django_q.models


class Migration(migrations.Migration):
    dependencies = [
        ("django_q", "0018_task_success_index"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="task",
            options={
                "ordering": ["-stopped"],
                "verbose_name": "Task",
                "verbose_name_plural": "Tasks",
            },
        ),
        migrations.AlterField(
            model_name="ormq",
            name="key",
            field=models.CharField(
                help_text="Name of the target cluster",
                max_length=100,
                verbose_name="Cluster key",
            ),
        ),
        migrations.AlterField(
            model_name="ormq",
            name="lock",
            field=models.DateTimeField(
                help_text="Prevent any cluster from pulling until",
                null=True,
                verbose_name="Lock until",
            ),
        ),
        migrations.AlterField(
            model_name="ormq",
            name="payload",
            field=models.TextField(
                help_text="Signed serialized task package (do not edit manually).",
                verbose_name="Payload",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="args",
            field=models.TextField(
                blank=True,
                help_text="e.g. 1, 2, 'John'",
                null=True,
                verbose_name="Arguments",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="cluster",
            field=models.CharField(
                blank=True,
                default=None,
                help_text="Name of the target cluster",
                max_length=100,
                null=True,
                verbose_name="Cluster",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="cron",
            field=models.CharField(
                blank=True,
                help_text="Cron expression",
                max_length=100,
                null=True,
                validators=[django_q.models.validate_cron],
                verbose_name="Cron",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="func",
            field=models.CharField(
                help_text="e.g. module.tasks.function",
                max_length=256,
                verbose_name="Function",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="hook",
            field=models.CharField(
                blank=True,
                help_text="e.g. module.tasks.result_function",
                max_length=256,
                null=True,
                verbose_name="Hook",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="intended_date_kwarg",
            field=models.CharField(
                blank=True,
                help_text="Name of kwarg to pass intended schedule date",
                max_length=100,
                null=True,
                validators=[django_q.models.validate_kwarg],
                verbose_name="Intended date kwarg",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="kwargs",
            field=models.TextField(
                blank=True,
                help_text="e.g. x=1, y=2, name='John'",
                null=True,
                verbose_name="Keyword arguments",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="minutes",
            field=models.PositiveSmallIntegerField(
                blank=True,
                help_text="Number of minutes for the Minutes type",
                null=True,
                verbose_name="Minutes",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="name",
            field=models.CharField(
                blank=True,
                help_text="Optional label to identify this schedule in the admin.",
                max_length=100,
                null=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="next_run",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                help_text="When this schedule runs next (stored in UTC).",
                null=True,
                verbose_name="Next Run",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="schedule_type",
            field=models.CharField(
                choices=[
                    ("O", "Once"),
                    ("I", "Minutes"),
                    ("H", "Hourly"),
                    ("D", "Daily"),
                    ("W", "Weekly"),
                    ("BW", "Biweekly"),
                    ("M", "Monthly"),
                    ("BM", "Bimonthly"),
                    ("Q", "Quarterly"),
                    ("Y", "Yearly"),
                    ("C", "Cron"),
                ],
                default="O",
                help_text="How often this task should be enqueued.",
                max_length=2,
                verbose_name="Schedule Type",
            ),
        ),
        migrations.AlterField(
            model_name="schedule",
            name="task",
            field=models.CharField(
                editable=False,
                help_text="Id of the last task spawned from this schedule (read-only).",
                max_length=100,
                null=True,
                verbose_name="Last task id",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="args",
            field=picklefield.fields.PickledObjectField(
                editable=False,
                help_text="Pickled positional arguments (tuple).",
                null=True,
                protocol=-1,
                verbose_name="Arguments",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="attempt_count",
            field=models.IntegerField(
                default=0,
                help_text="How many times execution was attempted.",
                verbose_name="Attempt count",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="cluster",
            field=models.CharField(
                blank=True,
                default=None,
                help_text="Name of the cluster that executed this task.",
                max_length=100,
                null=True,
                verbose_name="Cluster",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="func",
            field=models.CharField(
                help_text="Dotted import path to the callable, e.g. myapp.tasks.job.",
                max_length=256,
                verbose_name="Function",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="group",
            field=models.CharField(
                editable=False,
                help_text="Optional group id to aggregate related tasks.",
                max_length=100,
                null=True,
                verbose_name="Group",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="hook",
            field=models.CharField(
                help_text="Optional dotted path to a callback that receives the task result.",
                max_length=256,
                null=True,
                verbose_name="Hook",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="id",
            field=models.CharField(
                editable=False,
                help_text="32-character task identifier.",
                max_length=32,
                primary_key=True,
                serialize=False,
                verbose_name="Task id",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="kwargs",
            field=picklefield.fields.PickledObjectField(
                editable=False,
                help_text="Pickled keyword arguments (dict).",
                null=True,
                protocol=-1,
                verbose_name="Keyword arguments",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="name",
            field=models.CharField(
                editable=False,
                help_text="Optional human-readable name for lookup and display.",
                max_length=100,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="result",
            field=picklefield.fields.PickledObjectField(
                editable=False,
                help_text="Pickled return value or error payload.",
                null=True,
                protocol=-1,
                verbose_name="Result",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="started",
            field=models.DateTimeField(
                editable=False,
                help_text="When the worker started executing the task.",
                verbose_name="Started at",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="stopped",
            field=models.DateTimeField(
                editable=False,
                help_text="When the worker finished (success or failure).",
                verbose_name="Stopped at",
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="success",
            field=models.BooleanField(
                default=True,
                editable=False,
                help_text="False if the task raised an exception or timed out.",
                verbose_name="Success",
            ),
        ),
    ]



from multiprocessing.process import current_process
from multiprocessing.queues import Queue

from django import core, db
from django.apps.registry import apps
from django.utils.translation import gettext_lazy as _

try:
    apps.check_apps_ready()
except core.exceptions.AppRegistryNotReady:
    import django

    django.setup()

from django_q.brokers import Broker, get_broker
from django_q.conf import Conf, logger, setproctitle
from django_q.models import Success, Task
from django_q.signals import post_execute
from django_q.signing import SignedPackage
from django_q.tasks import async_chain
from django_q.utils import close_old_django_connections, get_func_repr

try:
    import setproctitle
except ModuleNotFoundError:
    setproctitle = None


def monitor(result_queue: Queue, broker: Broker = None):
    """
    Gets finished tasks from the result queue and saves them to Django
    :type broker: brokers.Broker
    :type result_queue: multiprocessing.Queue
    """
    if not broker:
        broker = get_broker()
    proc_name = current_process().name
    if setproctitle:
        setproctitle.setproctitle(f"qcluster {proc_name} monitor")
    logger.info(
        _("%(name)s monitoring at %(id)s")
        % {"name": proc_name, "id": current_process().pid}
    )
    for task in iter(result_queue.get, "STOP"):
        # save the result
        if task.get("cached", False):
            save_cached(task, broker)
        else:
            save_task(task, broker)
        # acknowledge result
        ack_id = task.pop("ack_id", False)
        if ack_id and (task["success"] or task.get("ack_failure", False)):
            broker.acknowledge(ack_id)
        # signal execution done
        post_execute.send(sender="django_q", task=task)
        # log the result
        info_name = get_func_repr(task["func"])
        if task["success"]:
            # log success
            logger.info(
                _("Processed '%(info_name)s' (%(task_name)s)")
                % {"info_name": info_name, "task_name": task["name"]}
            )
        else:
            # log failure
            logger.error(
                _("Failed '%(info_name)s' (%(task_name)s) - %(task_result)s")
                % {
                    "info_name": info_name,
                    "task_name": task["name"],
                    "task_result": task["result"],
                }
            )
    logger.info(_("%(name)s stopped monitoring results") % {"name": proc_name})


def save_task(task, broker: Broker):
    """
    Saves the task package to Django or the cache
    :param task: the task package
    :type broker: brokers.Broker
    """
    # SAVE LIMIT < 0 : Don't save success
    if not task.get("save", Conf.SAVE_LIMIT >= 0) and task["success"]:
        return
    # enqueues next in a chain
    if task.get("chain", None):
        async_chain(
            task["chain"],
            group=task["group"],
            cached=task["cached"],
            sync=task["sync"],
            broker=broker,
        )
    # SAVE LIMIT > 0: Prune database, SAVE_LIMIT 0: No pruning
    if not task.get("sync", False):
        close_old_django_connections()

    try:
        filters = {}
        if (
            Conf.SAVE_LIMIT_PER
            and Conf.SAVE_LIMIT_PER in {"group", "name", "func"}
            and Conf.SAVE_LIMIT_PER in task
        ):
            value = task[Conf.SAVE_LIMIT_PER]
            if Conf.SAVE_LIMIT_PER == "func":
                value = get_func_repr(value)
            filters[Conf.SAVE_LIMIT_PER] = value

        # check if we should clean the success tasks
        if Conf.SAVE_LIMIT > 0:
            with db.transaction.atomic(using=db.router.db_for_write(Success)):
                success_tasks_qs = Success.objects.filter(**filters)
                success_tasks_pks = [
                    success_task.pk
                    for success_task in success_tasks_qs.select_for_update()
                ]
                if task["success"] and len(success_tasks_pks) >= Conf.SAVE_LIMIT:
                    success_tasks_qs.last().delete()

        # check if this task has previous results
        try:
            task_obj = Task.objects.get(id=task["id"], name=task["name"])
            # only update the result if it hasn't succeeded yet
            if not task_obj.success:
                task_obj.stopped = task["stopped"]
                task_obj.result = task["result"]
                task_obj.success = task["success"]
                task_obj.attempt_count = task_obj.attempt_count + 1
                task_obj.save()

        except Task.DoesNotExist:
            # convert func to string
            func = get_func_repr(task["func"])
            task_obj = Task.objects.create(
                id=task["id"],
                name=task["name"],
                func=func,
                hook=task.get("hook"),
                args=task["args"],
                kwargs=task["kwargs"],
                cluster=task.get("cluster"),
                started=task["started"],
                stopped=task["stopped"],
                result=task["result"],
                group=task.get("group"),
                success=task["success"],
                attempt_count=1,
            )

        if (
            Conf.MAX_ATTEMPTS > 0
            and task_obj.attempt_count >= Conf.MAX_ATTEMPTS
            and task.get("ack_id")
        ):
            broker.acknowledge(task["ack_id"])

    except Exception:
        logger.exception("Could not save task result")


def save_cached(task, broker: Broker):
    task_key = f"{broker.list_key}:{task['id']}"
    timeout = task["cached"]
    if timeout is True:
        timeout = None
    try:
        group = task.get("group", None)
        iter_count = task.get("iter_count", 0)
        # if it's a group append to the group list
        if group:
            group_key = f"{broker.list_key}:{group}:keys"
            group_list = broker.cache.get(group_key) or []
            # if it's an iter group, check if we are ready
            if iter_count and len(group_list) == iter_count - 1:
                group_args = f"{broker.list_key}:{group}:args"
                # collate the results into a Task result
                results = [
                    SignedPackage.loads(broker.cache.get(k))["result"]
                    for k in group_list
                ]
                results.append(task["result"])
                task["result"] = results
                task["id"] = group
                task["args"] = SignedPackage.loads(broker.cache.get(group_args))
                task.pop("iter_count", None)
                task.pop("group", None)
                if task.get("iter_cached", None):
                    task["cached"] = task.pop("iter_cached", None)
                    save_cached(task, broker=broker)
                else:
                    save_task(task, broker)
                broker.cache.delete_many(group_list)
                broker.cache.delete_many([group_key, group_args])
                return
            # save the group list
            group_list.append(task_key)
            broker.cache.set(group_key, group_list, timeout)
            # async_task next in a chain
            if task.get("chain", None):
                async_chain(
                    task["chain"],
                    group=group,
                    cached=task["cached"],
                    sync=task["sync"],
                    broker=broker,
                )
        # save the task
        broker.cache.set(task_key, SignedPackage.dumps(task), timeout)
    except Exception:
        logger.exception("Could not save task result")



import time

from django.http import HttpResponse
from django.urls import reverse

from django_q import tasks


# internal function to be called with django_q
def new_task(run_for_minutes):
    print("Task started")
    time.sleep(run_for_minutes)
    print("Task done")
    return True


def add_task(request):
    task_id = tasks.async_task(new_task, 5)
    result_url = reverse("get_result", args=[task_id])
    return HttpResponse(
        f"Added async task with <a href='{result_url}'>Go to results</a>"
    )


def get_result(request, task_id):
    task = tasks.fetch(task_id)
    if not task:
        msg = "Task running... please refresh after some time"
    else:
        msg = f"Async task result: {task.result}"
    return HttpResponse(msg)



# Generated by Django 4.1.2 on 2023-01-15 22:34

from django.db import migrations, models

import django_q.models


class Migration(migrations.Migration):
    dependencies = [
        ("django_q", "0015_alter_schedule_schedule_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="schedule",
            name="intended_date_kwarg",
            field=models.CharField(
                blank=True,
                help_text="Name of kwarg to pass intended schedule date",
                max_length=100,
                null=True,
                validators=[django_q.models.validate_kwarg],
            ),
        ),
    ]



from iron_mq import IronMQ, Queue
from requests.exceptions import HTTPError

from django_q.brokers import Broker
from django_q.conf import Conf


class IronMQBroker(Broker):
    def enqueue(self, task):
        return self.connection.post(task)["ids"][0]

    def dequeue(self):
        timeout = Conf.RETRY or None
        tasks = self.connection.get(timeout=timeout, wait=1, max=Conf.BULK)["messages"]
        if tasks:
            return [(t["id"], t["body"]) for t in tasks]

    def ping(self) -> bool:
        return self.connection.name == self.list_key

    def info(self) -> str:
        return "IronMQ"

    def queue_size(self):
        return self.connection.size()

    def delete_queue(self):
        try:
            return self.connection.delete_queue()["msg"]
        except HTTPError:
            return False

    def purge_queue(self):
        return self.connection.clear()

    def delete(self, task_id):
        try:
            return self.connection.delete(task_id)["msg"]
        except HTTPError:
            return False

    def fail(self, task_id):
        self.delete(task_id)

    def acknowledge(self, task_id):
        return self.delete(task_id)

    @staticmethod
    def get_connection(list_key: str = None) -> Queue:
        list_key = list_key or Conf.CLUSTER_NAME
        ironmq = IronMQ(name=None, **Conf.IRON_MQ)
        return ironmq.queue(queue_name=list_key)



# Generated by Django 4.2.7 on 2024-03-05 17:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_q", "0017_task_cluster_alter"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="task",
            index=models.Index(
                condition=models.Q(("success", True)),
                fields=["group", "name", "func"],
                name="success_index",
            ),
        ),
    ]



"""Admin module for Django."""

from django.contrib import admin
from django.db.models.expressions import OuterRef, Subquery
from django.template.defaultfilters import truncatechars
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from django_q.conf import Conf, croniter
from django_q.models import Failure, OrmQ, Schedule, Success, Task
from django_q.tasks import async_task


def resubmit_task(model_admin, request, queryset):
    """Submit selected tasks back to the queue."""
    for task in queryset:
        async_task(
            task.func,
            *task.args or (),
            hook=task.hook,
            group=task.group,
            cluster=task.cluster,
            **task.kwargs or {},
        )
        if isinstance(model_admin, FailAdmin):
            task.delete()


resubmit_task.short_description = _("Resubmit selected tasks to queue")


class TaskAdmin(admin.ModelAdmin):
    """model admin for success tasks."""

    list_display = (
        "name",
        "group",
        "func",
        "cluster",
        "started",
        "stopped",
        "time_taken",
    )
    actions = [resubmit_task]

    def has_add_permission(self, request):
        """Don't allow adds."""
        return False

    def get_queryset(self, request):
        """Only show successes."""
        qs = super(TaskAdmin, self).get_queryset(request)
        return qs.filter(success=True)

    search_fields = ("name", "func", "group")
    readonly_fields = []
    list_filter = ("group", "cluster")

    def get_readonly_fields(self, request, obj=None):
        """Set all fields readonly."""
        return list(self.readonly_fields) + [field.name for field in obj._meta.fields]

    @admin.display(description=_("Time Taken"))
    def time_taken(self, obj):
        return obj.time_taken()


class FailAdmin(admin.ModelAdmin):
    """model admin for failed tasks."""

    list_display = (
        "name",
        "group",
        "func",
        "cluster",
        "started",
        "stopped",
        "short_result",
    )

    def has_add_permission(self, request):
        """Don't allow adds."""
        return False

    actions = [resubmit_task]
    search_fields = ("name", "func", "group")
    list_filter = ("group", "cluster")
    readonly_fields = []

    def get_readonly_fields(self, request, obj=None):
        """Set all fields readonly."""
        return list(self.readonly_fields) + [field.name for field in obj._meta.fields]

    @admin.display(description=_("Short result"))
    def short_result(self, obj):
        return truncatechars(obj.result, 100)


class ScheduleAdmin(admin.ModelAdmin):
    """model admin for schedules"""

    list_display = (
        "id",
        "name",
        "func",
        "schedule_type",
        "repeats",
        "cluster",
        "next_run",
        "get_last_run",
        "get_success",
    )

    # optional cron strings
    if not croniter:
        readonly_fields = ("cron",)

    list_filter = ("next_run", "schedule_type", "cluster")
    search_fields = (
        "name",
        "func",
    )
    list_display_links = ("id", "name")

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        task_query = Task.objects.filter(id=OuterRef("task")).values(
            "id", "name", "success"
        )
        qs = qs.annotate(
            task_id=Subquery(task_query.values("id")),
            task_name=Subquery(task_query.values("name")),
            task_success=Subquery(task_query.values("success")),
        )
        return qs

    def get_success(self, obj):
        return obj.task_success

    get_success.boolean = True
    get_success.short_description = _("success")

    def get_last_run(self, obj):
        if obj.task_name is not None:
            if obj.task_success:
                url = reverse("admin:django_q_success_change", args=(obj.task_id,))
            else:
                url = reverse("admin:django_q_failure_change", args=(obj.task_id,))
            return format_html('<a href="{}">[{}]</a>', url, obj.task_name)
        return None

    get_last_run.allow_tags = True
    get_last_run.short_description = _("last_run")


class QueueAdmin(admin.ModelAdmin):
    """queue admin for ORM broker"""

    list_display = ("id", "key", "name", "group", "func", "lock", "task_id")
    fields = (
        "key",
        "lock",
        "task_id",
        "name",
        "group",
        "func",
        "args",
        "kwargs",
        "q_options",
    )
    readonly_fields = fields[2:]

    def save_model(self, request, obj, form, change):
        obj.save(using=Conf.ORM)

    def delete_model(self, request, obj):
        obj.delete(using=Conf.ORM)

    def get_queryset(self, request):
        return super(QueueAdmin, self).get_queryset(request).using(Conf.ORM)

    def has_add_permission(self, request):
        """Don't allow adds."""
        return False

    list_filter = ("key",)


admin.site.register(Schedule, ScheduleAdmin)
admin.site.register(Success, TaskAdmin)
admin.site.register(Failure, FailAdmin)

if Conf.ORM or Conf.TESTING:
    admin.site.register(OrmQ, QueueAdmin)
